@@ -1,5 +1,7 @@
 /*
     Copyright (c) 2009 Stephen Kelly <steveire@gmail.com>
+    Copyright (c) 2016 Ableton AG <info@ableton.com>
+        Author Stephen Kelly <stephen.kelly@ableton.com>
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -1726,7 +1728,7 @@ void KSelectionProxyModelPrivate::removeSelectionFromProxy(const QItemSelection
 
     q->rootSelectionAboutToBeRemoved(selection);
 
-    Q_FOREACH (const QItemSelectionRange range, selection) {
+    Q_FOREACH (const QItemSelectionRange &range, selection) {
         removeRangeFromProxy(range);
     }
 }
@@ -2466,6 +2468,7 @@ void KSelectionProxyModel::setSelectionModel(QItemSelectionModel *itemSelectionM
                     SLOT(selectionChanged(QItemSelection,QItemSelection)));
 
             auto handleSelectionModelModel = [ &, d] {
+                beginResetModel();
                 if (d->selectionModelModelAboutToBeResetConnection)
                 {
                     disconnect(d->selectionModelModelAboutToBeResetConnection);
@@ -2482,7 +2485,11 @@ void KSelectionProxyModel::setSelectionModel(QItemSelectionModel *itemSelectionM
                     d->selectionModelModelResetConnection = connect(
                         d->m_selectionModel->model(),
                         SIGNAL(modelReset()), this, SLOT(sourceModelReset()));
+                    d->m_rootIndexList.clear();
+                    delete d->m_indexMapper;
+                    d->m_indexMapper = new KModelIndexProxyMapper(sourceModel(), d->m_selectionModel->model(), this);
                 }
+                endResetModel();
             };
 #if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
             connect(d->m_selectionModel.data(), &QItemSelectionModel::modelChanged,
@@ -2651,7 +2658,7 @@ QItemSelection KSelectionProxyModel::mapSelectionToSource(const QItemSelection &
                 const QModelIndex sourceIdx = mapToSource(idx);
                 Q_ASSERT(sourceIdx.isValid());
                 topSelection.append(QItemSelectionRange(sourceTopLeft, sourceIdx));
-                for (int i = it->top() + 1; i < it->bottom(); ++it) {
+                for (int i = it->top() + 1; i <= it->bottom(); ++i) {
                     const QModelIndex left = mapToSource(createIndex(i, 0));
                     const QModelIndex right = mapToSource(createIndex(i, it->right()));
                     Q_ASSERT(left.isValid());
